// triangulate.cpp
// MCS 481 Spring 2014 project 2 description version 1.0
#include <CGAL/Exact_predicates_exact_constructions_kernel.h>
#include <CGAL/Constrained_Delaunay_triangulation_2.h>
#include <CGAL/Partition_traits_2.h>
#include <CGAL/partition_2.h>
#include <CGAL/centroid.h>

#include <cassert>
#include <iostream>
#include <list>

typedef CGAL::Exact_predicates_exact_constructions_kernel K;

typedef CGAL::Triangulation_vertex_base_2<K>                     Vb;
typedef CGAL::Constrained_triangulation_face_base_2<K>           Fb;
typedef CGAL::Triangulation_data_structure_2<Vb,Fb>              TDS;
typedef CGAL::Exact_predicates_tag                               Itag;
typedef CGAL::Constrained_Delaunay_triangulation_2<K, TDS, Itag> CT;

typedef CGAL::Partition_traits_2<K>    Traits;
typedef CT::Point                      Point;
typedef CGAL::Triangle_2<K>            Triangle;
typedef Traits::Polygon_2              Polygon;
typedef Polygon::Edge_const_iterator   EdgeIterator;

int
main( )
{
  Polygon pgn;

  // Read vertex list from stdin and construct polygon
  Point p;
  while (std::cin >> p) {
    pgn.push_back(p);    
  }

  // We ask CGAL to triangulate the entire plane so that each edge of
  // P is one of the edges of this triangulation (these are the
  // "constraints").
  CT ct;
  EdgeIterator ei;
  for (ei = pgn.edges_begin(); ei != pgn.edges_end(); ei++) {
    ct.insert_constraint( (*ei).source(), (*ei).target() );
  }
  assert(ct.is_valid());

  // The CT object maintains a triangulation of the plane at all
  // times, adapting as needed it every time we add a constraint.  So
  // after the loop above, it includes a plane triangulation with P
  // among its edges.

  // Now we determine which triangles actually lie inside P and print
  // them.

  CT::Finite_faces_iterator fi;
  for (fi = ct.finite_faces_begin(); fi != ct.finite_faces_end(); fi++) {
    Triangle t = ct.triangle(fi);
    Point m = CGAL::centroid(t);
    if (pgn.bounded_side(m) == CGAL::ON_BOUNDED_SIDE) {
      // Current triangle is inside P, print it
      for (int i=0; i<3; i++) {
	std::cout << std::distance(pgn.vertices_begin(),
				    std::find(pgn.vertices_begin(),
					      pgn.vertices_end(),
					      t.vertex(i) ));
      	if (i==2) 
	  std::cout << std::endl;
      	else
	  std::cout << " ";
      }
    }
  }
  return 0;
}
